@@ -234,7 +234,7 @@ static void buildProbabilities(gpointer key, gpointer value, gpointer data)
 
     PINFO("P('%s') = '%d'\n", (char*)key, probability);
 
-    g_hash_table_insert(final_probabilities, key, (gpointer)probability);
+    g_hash_table_insert(final_probabilities, key, GINT_TO_POINTER(probability));
 }
 
 /* Frees an array of the same time that buildProperties built */
@@ -266,10 +266,10 @@ static void highestProbability(gpointer key, gpointer value, gpointer data)
   struct account_info *account_i = (struct account_info*)data;
 
   /* if the current probability is greater than the stored, store the current */
-  if((gint32)value > account_i->probability)
+  if(GPOINTER_TO_INT(value) > account_i->probability)
     {
       /* Save the new highest probability and the assoaciated account name */
-      account_i->probability = (gint32)value;
+      account_i->probability = GPOINTER_TO_INT(value);
       account_i->account_name = key;
     }
 }
@@ -476,6 +476,13 @@ void gnc_imap_add_account_bayes(GncImportMatchMap *imap, GList *tokens, Account
   for(current_token = g_list_first(tokens); current_token;
       current_token = current_token->next)
     {
+      /* Jump to next iteration if the pointer is not valid or if the
+	 string is empty. In HBCI import we almost always get an empty
+	 string, which doesn't work in the kvp loopkup later. So we
+	 skip this case here. */
+      if (!current_token->data || (*((char*)current_token->data) == '\0'))
+	continue;
+
       /* start off with no tokens for this account */
       token_count = 0;
 
