@@ -1,10 +1,11 @@
-;;;; $Id: gnumeric-utilities.scm,v 1.3 2001/05/15 16:33:56 rlb Exp $
+;;;; $Id: gnumeric-utilities.scm,v 1.4 2001/10/14 17:54:19 rlb Exp $
 ;;;; gnumeric-utilities.scm - Gnumeric spreadsheet generation functions
 
 (use-modules (srfi srfi-19))
 
-(gnc:support "gnumeric/gnumeric-utilities.scm")
-(gnc:depend "xml-generator.scm")
+;; (gnc:depend "xml-generator.scm") -- this needs to be changed to a
+;; use-modules, but since this file doesn't appear to be used right
+;; now, that can wait.
 
 ;;;; Gnumeric spreadsheet consists of:
 ;;;; gmr:Workbook
